#pragma once

#include "overlay_window.hpp"
#include "util.hpp"

#include <vc_controller.hpp>

#include <map>


class overlay_vc_controller : public vcsdk::vc_controller {
private:

	std::map<unsigned int, overlay_window> _windows;
	HINSTANCE _appInstance;

public:

	/// constructor.
	overlay_vc_controller(PVD vd, BYTE versionL, BYTE versionH, HINSTANCE hInstance)
		: vc_controller(vd, versionL, versionH)
		, _appInstance(hInstance) {
	}

	/// visitor function for overlay with a given id
	template <typename Fn>
	auto with_overlay(unsigned int id, Fn&& fn) {
		if (auto it = _windows.find(id); it != _windows.end()) {
			return fn(it->second);
		}
	}

	std::map<unsigned int, overlay_window>& get() {
		return _windows;
	}

	HINSTANCE appInstance() const { return _appInstance; }

	void add(unsigned int id, const std::wstring& title) {
		(void)overlay_window::emplace_new(_windows, id, title, _appInstance, get_viewport());
	}

	void remove(unsigned int id) {
		_windows.erase(id);
	}

	static overlay_vc_controller* from_handle(PVD pVd) {
		if (!pVd) throw vc_exception(make_vc_error_code(CLIENT_ERROR_NULL_MEM_POINTER));
		return reinterpret_cast<overlay_vc_controller*>(pVd->pPrivate);
	}

	template <typename T, typename... Args>
	void open_vc(const std::string& name, Args&&... args) {
		add_vc(std::unique_ptr<T>(new T(vc_controller::open_vc(name), name, this, std::forward<Args>(args)...)));
	}

};