#pragma once

#include "content.hpp"


class local_window {
public:
	unsigned int id() const { return 0; }
	void reposition(unsigned int x, unsigned int y) {}
	void resize(unsigned int dx, unsigned int dy) {}
	void clip(const window& win, clip_region& clip) {};
};



/// local application preview content
class application_preview : public display_content<local_window> {
private:

	window _win; /// win = target = capture source (ie. the application we want previews of)

public:

	/// constructor.
	application_preview(const window& win)
		: display_content((std::size_t)(HWND)win, local_window{})
		, _win(win) {
	}

	/// create a rectangle from dimensions, using {0, 0} as the origin.
	static constexpr RECT from_size(SIZE sz) {
		RECT rc = {};
		rc.right = sz.cx;
		rc.bottom = sz.cy;
		return rc;
	}

	/// generate a local preview image of the content.
	bitmap make_preview_image(const window& win, const SIZE& constraints) override {
		bitmap img = win.compatible_bitmap({ constraints.cx, constraints.cy });
		take_screenshot(img);
		return img;
	}

	/// take a screen shot of the target application window.
	void take_screenshot(bitmap& target) {

		// setup new bitmap object to receive window contents
		device_context::new_memory_dc().using_bitmap(_win.compatible_bitmap(), [this, &target](device_context& dc, bitmap& img) {

			// scrape window contents
			if (!_win.print(dc)) {
				dc.blt(_win.get_device_context(), { 0, 0 }, { 0, 0 }, img.dimensions());
			}

			// now scale it.
			device_context::new_memory_dc().using_bitmap(target, [&dc, &img](device_context& dc_scaled, bitmap& img_scaled) {
				dc_scaled.set_stretch_halftone();
				dc_scaled.stretch_blt(dc, from_size(img.dimensions()), from_size(img_scaled.dimensions()));
				});
			});
	}
};