/**********************************************************************
*
* ctxoverlay.cpp
*
* Example Virtual Channel application
*
* Copyright (C) Citrix Systems, Inc.  All Rights Reserved.
*
* The code and comments contained in this file are provided "as is"
* without warranty of any kind, either expressed or implied,
* including, but not limited to, the implied warranties of
* merchantability and/or fitness for a particular purpose.
* You have the right to use, modify, reproduce, and distribute
* this file, and/or any modified version of this file, in any way
* that you find useful, provided that you agree that Citrix Systems
* has no warranty obligations or liability for this or any other
* sample file.
*
**********************************************************************/

#include "pch.h"

#include <vdoverlay.h> // local header for CTXOVERLAY

#include "main_window.hpp"
#include "message_loop.hpp"

#include <locale>
#include <codecvt>
#include <fstream>


int APIENTRY wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow)
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

    wfsession session = wfsession::current_session(); // the remote session

    // open the virtual channel
    wfvirtualchannel::unique_resource overlay_vc = session.open_virtual_channel(CTXOVERLAY_VIRTUAL_CHANNEL_NAME);
    if (!overlay_vc) {
        MessageBox(NULL, L"Unable to open virtual channel", L"Error", MB_OK);
        return 0;
    }

    wfvirtualchannel::unique_resource appshare_vc = session.open_virtual_channel(CTXAPPSHARE_VIRTUAL_CHANNEL_NAME);
    if (!appshare_vc) {
        MessageBox(NULL, L"Unable to open appsharing virtual channel. App-sharing features will be unavailable.", L"Error", MB_OK);
    }

    InitCommonControls();

    auto win = main_window::create(hInstance, L"VC SDK Overlay Demo", session, overlay_vc.get(), appshare_vc.get());
    win.show();
    win.update();

    message_loop loop(LoadAccelerators(hInstance, MAKEINTRESOURCE(IDC_APP_ACCEL)));
    return loop.run();
}