#pragma once

#include "overlay_content.hpp"


class screen_sharing_session {
public:

	/// the session key type.
	using key_type = unsigned int;

	/// create a new session with the given session key and callback. flags are unused.
	static screen_sharing_session create(const key_type& key, session_callback_type&& callback, UINT32 flags) {
		screen_sharing_session session{ key, std::move(callback) };
		session.start();
		return session;
	}

	/// constructor. default construction.
	screen_sharing_session() = default;

private:

	/// constructor. construct with session key and callback.
	screen_sharing_session(const key_type& key, session_callback_type&& callback)
		: _key(key)
		, _callback(callback) {
	}

	/// start the session.
	void start() {
		// nothing to do server-side, so just fire the session started event.
		if (_callback) _callback(session_event::started, _key);
	}

	session_callback_type _callback; /// the callback to receive session events.
	key_type _key; /// the session key.
};


/// live screen-sharing content
using screen_content = overlayed_content<screen_sharing_session, WM_USER_SUBSCRIBESCREEN>;